/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.baseprogramming.dev.gen;

import com.baseprogramming.dev.FileUtil;
import com.baseprogramming.lang.WordCategory;
import com.baseprogramming.lang.Term;
import com.baseprogramming.lang.USCensusNameRecord;
import com.baseprogramming.lang.geography.City;
import com.baseprogramming.lang.geography.State;
import com.baseprogramming.lang.medical.DeceaseCause;
import com.baseprogramming.lang.medical.Symptom;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


/**
 *
 * @author benitez
 */
public final class DataFactory
{
  private static List<Term> TermList = new ArrayList<Term>();
  private static List<String> WordList= new ArrayList<String>();
  private static Map<WordCategory,List<Term>> wordCategories = new EnumMap<WordCategory,List<Term>>(WordCategory.class);
  private static List<State> States= new ArrayList<State>();
  private static List<City> Cities = new ArrayList<City>();
  private static List<Symptom> Symptoms = new ArrayList<Symptom>();
  private static List<DeceaseCause> DeseaseCauses= new ArrayList<DeceaseCause>();

  private DataFactory() 
  {
    
  }
  
  private static void addTerm(Term term)
  {
    TermList.add(term);
    WordList.add(term.getWord());
    for(WordCategory pos : term.getWordCategories())
    {
      List<Term> list=getWordCategoryTermList(pos);
      list.add(term);
    }
  }
  
  public static void loadAllDefaultData()
  {
    loadTerms();
    loadCities();
    loadStates();
    loadAnatomicalStructures();
    loadMedicalSymptoms();
    loadDeseaseCauses();
    loadMedicalTreatments();
    loadAllCensusNames();
    
  }
  
  public static void loadAllCensusNames()
  {
    try
    {
      readUSCensusNameFile(WordCategory.FIRST_NAME_FEMALE,FileUtil.FEMALE_NAMES);
      readUSCensusNameFile(WordCategory.FIRST_NAME_MALE,FileUtil.MALE_NAMES);
      readUSCensusNameFile(WordCategory.LAST_NAME,FileUtil.LAST_NAMES);
      
    }
    catch(Exception e)
    {
      String msg="Unexpected error while loading Census Names: " + e.getMessage();
      throw new java.lang.RuntimeException(msg);
    }
  }
  
  public static List<USCensusNameRecord> readUSCensusNameFile(WordCategory pos,String name) 
          throws IOException
  {
    List<USCensusNameRecord> records= new ArrayList<USCensusNameRecord>();
    
    InputStream is = null;
    BufferedReader br = null;
    
    try 
    { 
      is = FileUtil.class.getResourceAsStream(name);
      br = new BufferedReader(new InputStreamReader(is));
      while (true)
      {
         String line=br.readLine();
         if(line==null){break;}
         
          USCensusNameRecord record = USCensusNameRecord.parseFileRecord(line);
          Term word= new Term(record.getName(),pos);
          if(pos!=WordCategory.LAST_NAME)
          {
            word.addWordCategory(WordCategory.FIRST_NAME);
          }
          addTerm(word);
      }
    }
   
    finally
    {
        if (br != null) br.close();
        if (is != null) is.close();
    }
    
    return records;
    
    
  }
  
  public static void loadMedicalTreatments()
  {
    InputStream is = null;
    BufferedReader br = null;
    String name="/medical_treatment";
    
    try 
    { 
      is = DataFactory.class.getResourceAsStream(name);
      br = new BufferedReader(new InputStreamReader(is));
      loadMedicalTreatments(br);
    }
    catch(Exception e)
    {
      throw new java.lang.RuntimeException(e);
    }
   
    finally
    {
      try
      {
        if (br != null) br.close();
        if (is != null) is.close();
      }
      catch(Exception ignore){}
    }
  }
  
  public static  void loadMedicalTreatments(BufferedReader reader)
     throws java.io.IOException
  {
    WordCategory[] poss={WordCategory.MEDICAL_TREAMTENT,WordCategory.SIDE_EFFECT};
    
    String header=reader.readLine();
    while(true)
    {
      String line=reader.readLine();
      if(line==null){break;}
      String[] arr=line.split(",");
      int max=(arr.length>2)?2:1;
      for(int i=0;i<max;i++)
      {
        String token=arr[i];
        if(token.isEmpty()){continue;}
        Term word= new Term(token,poss[i]);
        addTerm(word);
      }
    }
  }
  
  public static void loadDeseaseCauses()
  {
    InputStream is = null;
    BufferedReader br = null;
    String name="/disease_cause";
    
    try 
    { 
      is = DataFactory.class.getResourceAsStream(name);
      br = new BufferedReader(new InputStreamReader(is));
      loadDeseaseCauses(br);
    }
    catch(Exception e)
    {
      throw new java.lang.RuntimeException(e);
    }
   
    finally
    {
      try
      {
        if (br != null) br.close();
        if (is != null) is.close();
      }
      catch(Exception ignore){}
    }
  }
  
  public static void loadDeseaseCauses(BufferedReader reader)
      throws java.io.IOException
  {
    DeseaseCauses.clear();
    String header=reader.readLine();
    while(true)
    {
      String line=reader.readLine();
      if(line==null){break;}
      DeceaseCause dc= DeceaseCause.parse(line);
      DeseaseCauses.add(dc);
      
      if(!dc.getName().isEmpty())
      {
        Term word=new Term(dc.getName(),WordCategory.DESEASE_CAUSE);
        addTerm(word);
      }
      
      if(!dc.getDeseaseCaused().isEmpty())
      {
        Term word= new Term(dc.getDeseaseCaused(),WordCategory.DESEASE);
        addTerm(word);
      }
    }
  }
  
  public static void loadMedicalSymptoms()
  {
    InputStream is = null;
    BufferedReader br = null;
    String name="/symptom";
    
    try 
    { 
      is = DataFactory.class.getResourceAsStream(name);
      br = new BufferedReader(new InputStreamReader(is));
      loadMedicalSymtoms(br);
    }
    catch(Exception e)
    {
      throw new java.lang.RuntimeException(e);
    }
   
    finally
    {
      try
      {
        if (br != null) br.close();
        if (is != null) is.close();
      }
      catch(Exception ignore){}
    }
  }
  
  public static void loadMedicalSymtoms(BufferedReader reader)
       throws java.io.IOException
  {
    Symptoms.clear();
    String header=reader.readLine();
    while(true)
    {
      String line=reader.readLine();
      if(line==null){break;}
      Symptom symptom= Symptom.parse(line);
      Symptoms.add(symptom);
      if(!(symptom.getName().isEmpty() ))
      {
        Term word= new Term(symptom.getName(),WordCategory.MEDICAL_SYMPTOM);
        addTerm(word);
      }
      
      if(!symptom.getSymtomOf().isEmpty())
      {
        Term word= new Term(symptom.getSymtomOf(),WordCategory.MEDICAL_SYMPTOM_CAUSE);
        addTerm(word);
      }
      
    }
  }
  
  public static void loadAnatomicalStructures()
  {
    InputStream is = null;
    BufferedReader br = null;
    String name="/anatomical_structure";
    
    try 
    { 
      is = DataFactory.class.getResourceAsStream(name);
      br = new BufferedReader(new InputStreamReader(is));
      loadAnatomicalStructures(br);
    }
    catch(Exception e)
    {
      throw new java.lang.RuntimeException(e);
    }
   
    finally
    {
      try
      {
        if (br != null) br.close();
        if (is != null) is.close();
      }
      catch(Exception ignore){}
    }
  }
  
  public static void loadAnatomicalStructures(BufferedReader reader)
     throws java.io.IOException
  {
     while(true)
      {
        String line=reader.readLine();
        if(line==null){break;}
        
        Term word= new Term(line,WordCategory.ANATOMICAL_STRUCTURE);
        word.addWordCategory(WordCategory.NOUN);
        addTerm(word);
      }
  }
  
  public static void loadStates()
  {
    String name="/states.txt";
    InputStream is = null;
    BufferedReader br = null;
    
    try 
    { 
      is = DataFactory.class.getResourceAsStream(name);
      br = new BufferedReader(new InputStreamReader(is));
      loadStates(br);
    }
    catch(Exception e)
    {
      throw new java.lang.RuntimeException(e);
    }
   
    finally
    {
      try
      {
        if (br != null) br.close();
        if (is != null) is.close();
      }
      catch(Exception ignore){}
    }
  }
  
  public static void loadCities()
  {
    String name="/cities_america.csv";
    InputStream is = null;
    BufferedReader br = null;
    
    try 
    { 
      is = DataFactory.class.getResourceAsStream(name);
      br = new BufferedReader(new InputStreamReader(is));
      loadCities(br);
    }
    catch(Exception e)
    {
      throw new java.lang.RuntimeException(e);
    }
   
    finally
    {
      try
      {
        if (br != null) br.close();
        if (is != null) is.close();
      }
      catch(Exception ignore){}
    }
  }
  
  public static void loadCities(BufferedReader reader)
      throws java.io.IOException
  {
      States.clear();
    
     while (true)
      {
        String line=reader.readLine();
         if(line==null){break;}
         City city= City.parse(line);
         Cities.add(city);
         
         Term word= new Term(city.getName(),WordCategory.PROPER_NOUN);
         word.addWordCategory(WordCategory.CITY);
         addTerm(word);
      }
  }
  
  public static void loadStates(BufferedReader reader)
      throws java.io.IOException
  {
      States.clear();
    
     while (true)
      {
        String line=reader.readLine();
         if(line==null){break;}
         State state= State.parse(line);
         States.add(state);
         
         Term word= new Term(state.getName(),WordCategory.PROPER_NOUN);
         word.addWordCategory(WordCategory.STATE);
         addTerm(word);
      }
  }
  
  public static void loadTerms()
  {
    String name="/part-of-speech.txt";
    InputStream is = null;
    BufferedReader br = null;
    
    try 
    { 
      is = DataFactory.class.getResourceAsStream(name);
      br = new BufferedReader(new InputStreamReader(is));
     loadTerms(br);
    }
    catch(Exception e)
    {
      throw new java.lang.RuntimeException(e);
    }
   
    finally
    {
      try
      {
        if (br != null) br.close();
        if (is != null) is.close();
      }
      catch(Exception ignore){}
    }
  }
  
   public static void loadTerms(BufferedReader reader)
       throws java.io.IOException
  {
    TermList.clear();
    wordCategories.clear();
    while(true)
    {
      String line=reader.readLine();
      if(line==null){break;}
      Term word=Term.createFromTabDelimitedString(line);
      addTerm(word);
    }
  }
   
   /**
    * Returns a male first name {@link  Term}. Use the <code>getWord()</code> method to get the string.
    * * <p>
    * Use the {@link #loadAllCensusNames() } method to load the names first.
    * </p>
    * @return  Term
    */
   public static Term getMaleFirstName()
   {
     return getTerm(WordCategory.FIRST_NAME_MALE);
   }

   /**
    * Returns a female first name {@link  Term}. Use the <code>getWord()</code> method to get the string.
    * * <p>
    * Use the {@link #loadAllCensusNames() } method to load the names first.
    * </p>
    * @return  Term
    */
   public static Term getFemaleFirstName()
   {
     return getTerm(WordCategory.FIRST_NAME_FEMALE);
   }
   
   /**
    *  Returns a First Name {@link Term}.  Use the <code>getWord()</code> method to get the string. The name could be male or female.
    * <p>
    * Use the {@link #loadAllCensusNames() } method to load the names first.
    * </p>
    * @return Term
    */
   public static Term getFirstNameAnyGender()
   {
     return getTerm(WordCategory.FIRST_NAME);
   }
   
   /**
    * Returns a Last Name {@link Term}.  Use the <code>getWord()</code> method to get the string
    * <p>
    * Use the {@link #loadAllCensusNames() } method to load the names first.
    * </p>
    * @return Term
    */
   public static Term getLastName()
   {
     return getTerm(WordCategory.LAST_NAME);
   }
   
   /**
    * Returns a {@link DeceaseCause}
    * @return DeseaseCause
    */
   public static DeceaseCause getDeseaseCause()
   {
     if(DeseaseCauses.isEmpty()){loadDeseaseCauses();}
     Random rand= new Random();
     int index=rand.nextInt(DeseaseCauses.size()-1);
     return DeseaseCauses.get(index);
   }
   
   /**
    * Returns a {@link  Symptom} 
    * @return Symptom
    */
   public static Symptom getMedicalSymptom()
   {
     if(Symptoms.isEmpty())
     {
       if(TermList.isEmpty()){loadTerms();}
       loadMedicalSymptoms();
     }
     Random random= new Random();
     int rand=random.nextInt(Symptoms.size()-1);
     return Symptoms.get(rand);
   }
   
   /**
    * Returns a medical symptom name
    * @return String 
    */
   public static String getMedicalSymptomName()
   {
     return getMedicalSymptom().getName();
   }
   
   /**
    *Returns the name of an anatomical structure
    * @return String
    */
   public static String getAnatomicalStructure()
   {
     if(TermList.isEmpty())
     {
       loadTerms();
       loadAnatomicalStructures();
     }
     return getTerm(WordCategory.ANATOMICAL_STRUCTURE).getWord();
   }
   
   /**
    * Returns a location name: {@link City} or {@link State}
    * @return String
    */
   public static String getLocationName()
   {
     Random random= new Random();
     int rand=random.nextInt(100);
     if(rand<50){return getCity().getName();}
     else{return getState().getName();}
   }
  
   /**
    * Returns a {@link  City}
    * @return City
    */
   public static City getCity()
  {
    if(Cities.isEmpty()){loadCities();}
    if(Cities.isEmpty()){return null;}
    else
    {
      Random random= new Random();
      int index=random.nextInt(Cities.size())-1;
      return Cities.get(index);
    }
  }
   
  /**
   * Returns a {@link State}
   * @return State
   */
  public static State getState()
  {
    if(States.isEmpty()){loadStates();}
    if(States.isEmpty()){return null;}
    else
    {
      Random random= new Random();
      int index=random.nextInt(States.size())-1;
      return States.get(index);
    }
  }
  
  /**
   * Returns a {@link Term}
   * @return {@link  Term}
   */
  public static Term getTerm()
  {
    if(TermList.isEmpty()){loadTerms();}
    
    return getTerm(TermList);
  }
  
  /**
   * Returns a {@link Term} for the given category
   * @param pos
   * @return {@link  Term}
   */
  public static Term getTerm(WordCategory pos)
  {
    if(wordCategories.isEmpty()){loadTerms();}
    List<Term> list=getWordCategoryTermList(pos);
    return getTerm(list);
  }
  
  private  static List<Term> getWordCategoryTermList(WordCategory pos)
  {
    List<Term> list;
    if(!wordCategories.containsKey(pos ) )
    {
      list=new ArrayList<Term>();
      wordCategories.put(pos, list);
    }
    else
    {
      list=wordCategories.get(pos);
    }
    
    return list;
  }
  
  private static Term getTerm(List<Term> list)
  {
    if(list.isEmpty())
    {
      String msg="The word list is empty.  Make sure to load a word list first.  call method loadAllDefaultData() to load all word lists contained in jar file.  Otherwise, load your own word list for the category required.";
      throw new java.lang.IllegalStateException(msg);
    }
    Random rand= new Random();
    int index=rand.nextInt(list.size()-1);
    
    return list.get(index);
  }
  
  /***
   * Process Jav Lib text.  variables are expected to be in the form
   * <b>@{<enum constant of class com.baseprogramming.lang.WordCategory>}</b>
   * <p>
   * <b>Example:</b>
   * </p>
   * <pre>
   *      Hello.  My name is @{FIRST_NAME_MALE} @{LAST_NAME}.
   * </pre>
   * @param inputText
   * @return String
   */
  public static String processJavLib(String inputText)
  {
    if(TermList.isEmpty())
    {
      DataFactory.loadAllDefaultData();
    }
    String result=inputText;
    String regex="@\\{[a-zA-Z]+(_[a-zA-Z]+)*\\}";
    Pattern pattern=Pattern.compile(regex);
    Matcher matcher=pattern.matcher(inputText);
    
    while(matcher.find())
    {
      int start=matcher.start();
      int end=matcher.end();
      String var=inputText.substring(start, end);
      WordCategory pos=WordCategory.valueOf(var.replaceAll("[@\\{\\}]", ""));
      String word=DataFactory.getTerm(pos).getWord();
      String varRegEx="@\\{" + pos  + "\\}";
      result=result.replaceFirst(varRegEx, word );
    }
    return result;
  }
  
  /**
   * Returns <code>count</code> words delimited by spaces
   * @param count
   * @return String
   */
  public static String genWords(int count)
  {
    StringBuilder builder= new StringBuilder();
    for(int i=0;i<count;i++)
    {
      String word=getTerm().getWord() + " ";
      builder.append(word);
    }
    
    return builder.toString();
  }
  
  /**
   * Generates a string of size <code>maxlength</code>, consisting of space delimited words.
   * @param maxlength
   * @return String
   */
  public static String genString(int maxlength)
  {
    StringBuilder builder= new StringBuilder();
    while(true)
    {
      String word=getTerm().getWord()+" ";
      int remaining=maxlength-builder.length();
      
      if(remaining>word.length())
      {
          builder.append(word);
      }
      else
      {
          String string=word.substring(0, remaining);
          builder.append(string);
          break;
      }    
      
    }
    
    return builder.toString();
  }
  
  /**
   * Generates an integer between <code>min</code> and <code>max</code>
   * @param min
   * @param max
   * @return integer
   */
  public static int genNumberInRange(int min, int max)
  {
    return min + (int)Math.round(Math.random() * (max - min));

  }
  
  /**
   * Generates a date
   * @return {@link Date}
   */
  public static java.util.Date genDate()
  {
    GregorianCalendar gc= new GregorianCalendar();
    gc.setTime(new java.util.Date());
    int max=gc.get(Calendar.YEAR);
    
    return genDate(1970,max);
  }
  
  /**
   * Returns a Date between specified years.
   * @param minYear
   * @param maxYear
   * @return Date
   */
  public static java.util.Date genDate(int minYear,int maxYear)
  {
    GregorianCalendar gc= new GregorianCalendar();
    
    int year=genNumberInRange(minYear,maxYear);
    int month=genNumberInRange(0,11);
    
    gc.set(year, month, 1);
    int maxDay=gc.getActualMaximum(Calendar.DAY_OF_MONTH);
    
    int day=genNumberInRange(1,maxDay);
    int hour=DataFactory.genNumberInRange(0, 24);
    int min=DataFactory.genNumberInRange(0, 60);
    int sec=DataFactory.genNumberInRange(0, 60);
    
    gc.set(year, month, day,hour,min,sec);
   
   
   return gc.getTime();
  }
  
  /**
   * Returns a random date between <code>start</bode> and <code>end</code>
   * @param start
   * @param end
   * @return Date
   */
  public static java.util.Date genDateBetween(java.util.Date start, java.util.Date end)
    {
        GregorianCalendar gc= new GregorianCalendar();
        gc.setTime(start);
        int minYear=gc.get(Calendar.YEAR);
        int minMonth=gc.get(Calendar.MONTH);
        int minDay=1;
        gc.setTime(end);
        int maxYear=gc.get(Calendar.YEAR);
        int maxMonth=gc.get(Calendar.MONDAY);
        int maxDay=gc.get(Calendar.DAY_OF_MONTH);
        
        int year=DataFactory.genNumberInRange(minYear, maxYear);
        
        int month=DataFactory.genNumberInRange(minMonth, maxMonth);
        if(month!=minMonth)
        {
            minDay=gc.get(Calendar.DAY_OF_MONTH);
        }
        if(month!=maxMonth)
        {
            maxDay=getDaysInMonth(month, year);
        }

        int day=DataFactory.genNumberInRange(minDay, maxDay);
        int hour=DataFactory.genNumberInRange(0, 24);
        int min=DataFactory.genNumberInRange(0, 60);
        int sec=DataFactory.genNumberInRange(0, 60);
        gc.set(year, month, day,hour,min,sec);
        
        return gc.getTime();
    }
  
    private static int getDaysInMonth(int month,int year)
    {
        if(month<0 || month>11)
        {
            String msg="make sure to pass a 0-based month index--for the sake of conformity with java Date-APIs";
            throw new java.lang.IndexOutOfBoundsException(msg);
        }
        int [] daysInMonths = {31,28,31,30,31,30,31,31,30,31,30,31};
        daysInMonths[1] +=isLeapYear(year)==true ? 1 : 0;
        return daysInMonths[month];
    }
  
    /**
     * Returns weather the integer corresponds to a leap year 
     * @param year
     * @return boolean
     */
    public static boolean isLeapYear(int year)
    {
        if( year % 400==0) {return true;}
        else if (year % 100==0) {return false;}
        else if  (year % 4==0) {return true;}
        else {return false;}
    }
  
    /**
     * Returns a random element from <code>list</code>
     * @param <T>
     * @param list
     * @return T
     */
  public static <T> T selectFromList(List<T> list)
  {
    if(list==null || list.isEmpty()){return null;}
    
    Random rand= new Random();
    int index=rand.nextInt(list.size());
    return list.get(index);
  }
  
  /**
   * Returns true if the word exists in the <bold>loaded</bold> dictionaries, false otherwise.
   * @param word
   * @return boolean.
   */
  public static boolean exists(String word)
  {
      if(WordList.isEmpty()){DataFactory.loadAllDefaultData();}
      return WordList.contains(word);
  }
  
}
