/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.baseprogramming.lang;

/**
 *
 * @author benitez
 */
public enum WordCategory
{
  /***
   * A word (other than a pronoun) used to identify any of a class of people, places, or things (common noun), or to name a particular one of these (proper noun)
   */
  NOUN("N"),
  /***
   * The form of a word that is used to denote more than one
   */
  PLURAL("p"),
  
  /***
   * A word or group of words that functions in a sentence as subject, object, or prepositional object
   */
  NOUN_PHRASE("h"),
  
  /***
   * A word used to describe an action, state, or occurrence, and forming the main part of the predicate of a sentence, such as hear, become, happen
   */
  VERB("V"),
  
  /***
   * a verb (or verb construction) that requires an object in order to be grammatical
   */
  TRANSITIVE_VERB("t"),
  
  /***
   * A verb that is associated with only one noun or noun phrase.
   */
  INTRANSITIVE_VERB("i"),
  
  /***
   * A word or phrase naming an attribute, added to or grammatically related to a noun to modify or describe it


   */
  ADJETIVE("A"),
  
  /***
   * A word or phrase that modifies or qualifies an adjective, verb, or other adverb or a word-group, expressing a relation of place, time, circumstance, manner, cause, degree, etc. (e.g., gently, quite, then, there)
   */
  ADVERB("v"),
  
  /***
   * A word used to connect clauses or sentences or to coordinate words in the same clause (e.g., and, but, if)
   */
  CONJUNCTION("C"),
  
  /***
   * A word governing, and usually preceding, a noun or pronoun and expressing a relation to another word or element in the clause, as in “the man on the platform,” “she arrived after dinner,” “what did you do it for?”
   */
  PREPOSITION("P"),
  
  /***
   * An exclamation, e.g., ah! or dear me!
   */
  INTERJECTION("!"),
  
  /***
   * A word that can function by itself as a noun phrase and that refers either to the participants in the discourse (e.g., I, you) or to someone or something mentioned elsewhere in the discourse (e.g., she, it, this)
   */
  PRONOUN("r"),
  
  /***
   * A determiner (the in English) that introduces a noun phrase and implies that the thing mentioned has already been mentioned, or is common knowledge, or is about to be defined (as in the book on the table; the art of government; the famous poet and short story writer)
   */
  DEFINITE_ARTICLE("D"),
  
  /***
   * A determiner (a and an in English) that introduces a noun phrase and implies that the thing referred to is nonspecific (as in she bought me a book; government is an art; he went to a public school). Typically, the indefinite article is used to introduce new concepts into a discourse
   */
  INDEFINITE_ARTICLE("I"),
  
  /***
   * Relating to or denoting a case of nouns, pronouns, and adjectives (as in Latin and other inflected languages) used for the subject of a verb
   */
  NOMINATIVE("o"),
  
  /***
   * A name used for an individual person, place, or organization, spelled with initial capital letters, e.g., Larry, Mexico, and Boston Red Sox
   */
  PROPER_NOUN("u"),
  
  FIRST_NAME("fn"),
  FIRST_NAME_MALE("fnm"),
  FIRST_NAME_FEMALE("fnf"),
  LAST_NAME("ln"),
  CITY("city"),
  STATE("state"),
  ANATOMICAL_STRUCTURE("anatomy"),
  MEDICAL_SYMPTOM("symptom"),
  MEDICAL_SYMPTOM_CAUSE("symptomcause"),
  SIDE_EFFECT("sideeffect"),
  DESEASE("desease"),
  DESEASE_CAUSE("deseasecause"),
  MEDICAL_TREAMTENT("medicaltreatment");

  private final String Id;
  
  WordCategory(String id)
  {
    Id=id;
  }

  public String getId() {
    return Id;
  }
  
  public static WordCategory forId(String id)
  {
    String msg="[" + id + "] is not a valid Id for PartOfSpeach. Valid IDs are:  ";
    for(WordCategory pos : WordCategory.values())
    {
      if(pos.getId().equals(id))
      {
        return pos;
      }
    }
    throw new java.lang.IllegalArgumentException(msg + getValidIds());
  }
    
    private static  String getValidIds()
    {
      StringBuilder Ids= new StringBuilder();
      for(WordCategory pos : WordCategory.values())
      {
        if(Ids.length()>0){Ids.append(", ");}
        Ids.append(pos.getId());
      }
        
      return Ids.toString();
    }
    
  
}
